﻿// ==============================================================
//                 ORBITER MODULE: JBT_B1200
//                  Part of the ORBITER SDK
//					Tristan Lostroh
//JBT_B1200.cpp
// Control module for JBT_B1200 vessel class
// Handles the vessel configuration
// Does not handle the towing code
// Notes:
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include <math.h>
#include <string.h>
#include "JBT_B1200.h"

//UMMU
#include "UmmuSDK.h" 

//////////////////////////////////////////////////////////////////////////////////////////////
// 1-ORBITERSOUND EXAMPLE - INCLUDE SDK FILES 
//////////////////////////////////////////////////////////////////////////////////////////////
#include "OrbiterSoundSDK40.h"


JBT_B1200::JBT_B1200(OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
	complex = flightmodel;
	attachAction = false;
	
	static VECTOR3 LFWHEEL_REF  = {-1.065,0.75,1.83};
	static VECTOR3 RFWHEEL_REF  = {1.065,0.75,1.83};
	static VECTOR3 LRWHEEL_REF  = {-1.065,0.75,-1.83};
	static VECTOR3 RRWHEEL_REF  = {1.065,0.75,-1.83};

	static UINT GRP_LFWHEEL[1] = {0};
	static UINT GRP_LFWHEELP[1] = {};
	static UINT GRP_RFWHEEL[1] = {2};
	static UINT GRP_RFWHEELP[1] = {};
	static UINT GRP_LRWHEEL[1] = {1};
	static UINT GRP_LRWHEELP[1] = {};
	static UINT GRP_RRWHEEL[1] = {3};
	static UINT GRP_RRWHEELP[1] = {};

	// Steering animation transformation definitions
	static VECTOR3 LWHEEL_AXIS = {0,1,0};
	static VECTOR3 RWHEEL_AXIS = {0,1,0};
	static float INSIDESTEERING_RANGE_F = (float)(B1200_TURNINGANGLEINSIDE_F);
	static float OUTSIDESTEERING_RANGE_F = (float)(B1200_TURNINGANGLEOUTSIDE_F);
	static float INSIDESTEERING_RANGE_R = (float)(B1200_TURNINGANGLEINSIDE_R);
	static float OUTSIDESTEERING_RANGE_R = (float)(B1200_TURNINGANGLEOUTSIDE_R);

	// -5.73228E-06 0.766019 -0.642818 0 0
	static VECTOR3 SPEEDODIAL_REF	= {-0.48,1.43437,4.12454};
	static VECTOR3 REVDIAL_REF		= {-0.72,1.43437,4.12455};
	static VECTOR3 FUELDIAL_REF		= {-0.6,1.39581,4.07858};
	static VECTOR3 GEARDIAL_REF		= {-0.62,1.4855,4.18608};
	static VECTOR3 THR_REF			= {-0.448,0.609,3.941};
	static VECTOR3 BRK_REF			= {-0.748,0.609,3.941};
	static VECTOR3 STR_REF			= {-0.6,1.37,3.787};
	static VECTOR3 PRK_REF			= {-0.037,1.048,3.464};
	static VECTOR3 GRSTK_REF		= {-0.146,0.933,3.700 };
	static VECTOR3 WARN_REF		= {-0.9,1.4,4.077};
	static VECTOR3 HEAD_REF		= {-0.9,1.444,4.137};
	static VECTOR3 COMPLEX_REF		= {-0.044,1.335,3.975};
	static VECTOR3 SM_REF		= {-0.143891,1.25191,3.90956};

	static UINT GRP_SPEEDODIAL[1] = {4};
	static UINT GRP_REVDIAL[1] = {5};
	static UINT GRP_FUELDIAL[1] = {3};
	static UINT GRP_GEARDIAL[1] = {6};
	static UINT	GRP_THR[1] = {1};
	static UINT	GRP_BRK[1] = {2};
	static UINT	GRP_STR[1] = {10};
	static UINT	GRP_PRK[1] = {20};
	static UINT	GRP_GRSTK[1] = {19};
	static UINT	GRP_WARN[1] = {12};
	static UINT	GRP_HEAD[1] = {11};
	static UINT	GRP_CPLX[1] = {13};
	static UINT	GRP_SM[1] = {22};
	static UINT	GRP_FUELBTN[1] = {21};
	static UINT	GRP_BUTTON[4][1] = {{16},{15},{14},{17}};

	static VECTOR3 DASH_ANGLE = {0,0.766019,-0.642818};
	static VECTOR3 BUTTON_DISTANCE = {0,0.0091922,-0.0077138};

	//50Degrees
	static VECTOR3 STR_ANGLE = {0,-0.642818,0.76604};

	//Wheel Left Rear
	LeftRear = new MGROUP_ROTATE( 
		1,                          // mesh index 
		GRP_LRWHEEL, 1,            // group list and # groups 
		LRWHEEL_REF,             // rotation reference point 
		_V(1,0,0),                  // rotation axis 
		(float)(2*PI)               // angular rotation range 
	); 

	//Wheel Right Rear
	RightRear = new MGROUP_ROTATE( 
		1,                          // mesh index 
		GRP_RRWHEEL, 1,            // group list and # groups 
		RRWHEEL_REF,             // rotation reference point 
		_V(1,0,0),                  // rotation axis 
		(float)(2*PI)               // angular rotation range 
	); 


	//Wheel Left Front
	LeftFront = new MGROUP_ROTATE( 
		1,                          // mesh index 
		GRP_LFWHEEL, 1,            // group list and # groups 
		LFWHEEL_REF,             // rotation reference point 
		_V(1,0,0),                  // rotation axis 
		(float)(2*PI)               // angular rotation range 
	); 

	//Wheel Right Front
	RightFront = new MGROUP_ROTATE( 
		1,                          // mesh index 
		GRP_RFWHEEL, 1,            // group list and # groups 
		RFWHEEL_REF,             // rotation reference point 
		_V(1,0,0),                  // rotation axis 
		(float)(2*PI)               // angular rotation range 
	); 

	//Wheel Left Front Steer Right
	static MGROUP_ROTATE  LeftFrontSteer( 
		1,                          // mesh index 
		GRP_LFWHEELP, 0,            // group list and # groups 
		LFWHEEL_REF,             // rotation reference point 
		_V(0,1,0),                  // rotation axis 
		-INSIDESTEERING_RANGE_F-OUTSIDESTEERING_RANGE_F               // angular rotation range 
	); 

	//Wheel Right Front Steer Right
	static MGROUP_ROTATE RightFrontSteer( 
		1,                          // mesh index 
		GRP_RFWHEELP, 0,            // group list and # groups 
		RFWHEEL_REF,             // rotation reference point 
		_V(0,1,0),                  // rotation axis 
		OUTSIDESTEERING_RANGE_F+INSIDESTEERING_RANGE_F              // angular rotation range 
	); 

	//Wheel Left Rear Steer Right
	static MGROUP_ROTATE LeftRearSteer( 
		1,                          // mesh index 
		GRP_LRWHEELP, 0,            // group list and # groups 
		LRWHEEL_REF,             // rotation reference point 
		_V(0,1,0),                  // rotation axis 
		INSIDESTEERING_RANGE_R+OUTSIDESTEERING_RANGE_R               // angular rotation range 
	); 

	//Wheel Right Rear Steer Right
	static MGROUP_ROTATE RightRearSteer( 
		1,                          // mesh index 
		GRP_RRWHEELP, 0,            // group list and # groups 
		RRWHEEL_REF,             // rotation reference point 
		_V(0,1,0),                  // rotation axis 
		-OUTSIDESTEERING_RANGE_R-INSIDESTEERING_RANGE_R              // angular rotation range 
	); 
	// Steering animations

	//VC Animations
	//Speedo
	static MGROUP_ROTATE  Speedo_VC( 
		0,                          // mesh index 
		GRP_SPEEDODIAL, 1,            // group list and # groups 
		SPEEDODIAL_REF,             // rotation reference point 
		DASH_ANGLE,                  // rotation axis 
		(float) PI              // angular rotation range 
	); 
	//Rev Counter
	static MGROUP_ROTATE  Rev_VC( 
		0,                          // mesh index 
		GRP_REVDIAL, 1,            // group list and # groups 
		REVDIAL_REF,             // rotation reference point 
		DASH_ANGLE,                  // rotation axis 
		(float) (150*RAD)              // angular rotation range 
	); 
	//Fuel
	static MGROUP_ROTATE  Fuel_VC( 
		0,                          // mesh index 
		GRP_FUELDIAL, 1,            // group list and # groups 
		FUELDIAL_REF,             // rotation reference point 
		DASH_ANGLE,                  // rotation axis 
		(float) (120*RAD)              // angular rotation range 
	); 
	static MGROUP_TRANSLATE  Gear_VC_Left( 
		0,                          // mesh index 
		GRP_GEARDIAL, 1,            // group list and # groups 
		_V(0.02,0,0)              // tranlation vector
	); 
	static MGROUP_TRANSLATE  Gear_VC_Right( 
		0,                          // mesh index 
		GRP_GEARDIAL, 1,            // group list and # groups 
		_V(0.045,0,0)              // tranlation vector
	); 

	static MGROUP_ROTATE  Throttle_VC( 
		0,                          // mesh index 
		GRP_THR, 1,            // group list and # groups 
		THR_REF,             // rotation reference point 
		_V(1,0,0),                  // rotation axis 
		(float) (25*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  Brake_VC( 
		0,                          // mesh index 
		GRP_BRK, 1,            // group list and # groups 
		BRK_REF,             // rotation reference point 
		_V(1,0,0),                  // rotation axis 
		(float) (25*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  Steer_VC( 
		0,                          // mesh index 
		GRP_STR, 1,            // group list and # groups 
		STR_REF,             // rotation reference point 
		STR_ANGLE,                  // rotation axis 
		(float) (450*RAD)              // angular rotation range 
	); 

	static MGROUP_ROTATE  Park_VC( 
		0,                          // mesh index 
		GRP_PRK, 1,            // group list and # groups 
		PRK_REF,             // rotation reference point 
		_V(-1,0,0),                  // rotation axis 
		(float) (25*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  GearStick_VC( 
		0,                          // mesh index 
		GRP_GRSTK, 1,            // group list and # groups 
		GRSTK_REF,             // rotation reference point
		_V(-1,0,0),                  // rotation axis 
		(float) (30*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  Warninglights_VC( 
		0,                          // mesh index 
		GRP_WARN, 1,            // group list and # groups 
		WARN_REF,             // rotation reference point
		_V(1,0,0),                  // rotation axis 
		(float) (25*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  Headlights_VC( 
		0,                          // mesh index 
		GRP_HEAD, 1,            // group list and # groups 
		HEAD_REF,             // rotation reference point
		_V(1,0,0),                  // rotation axis 
		(float) (25*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  Complex_VC( 
		0,                          // mesh index 
		GRP_CPLX, 1,            // group list and # groups 
		COMPLEX_REF,             // rotation reference point
		_V(1,0,0),                  // rotation axis 
		(float) (25*RAD)              // angular rotation range 
	); 
	static MGROUP_ROTATE  SteerMode_VC( 
		0,                          // mesh index 
		GRP_SM, 1,            // group list and # groups 
		SM_REF,             // rotation reference point
		_V(1,0,0),                  // rotation axis 
		(float) (40*RAD)              // angular rotation range 
	); 

	static MGROUP_TRANSLATE FuelBtn_VC( 
		0,                          // mesh index 
		GRP_FUELBTN, 1,            // group list and # groups 
		BUTTON_DISTANCE              // tranlation vector
	);

	static MGROUP_TRANSLATE  Buttons_VC[4] = {
		MGROUP_TRANSLATE( 
			0,                          // mesh index 
			GRP_BUTTON[0], 1,            // group list and # groups 
			BUTTON_DISTANCE              // tranlation vector
		),
		MGROUP_TRANSLATE( 
			0,                          // mesh index 
			GRP_BUTTON[1], 1,            // group list and # groups 
			BUTTON_DISTANCE              // tranlation vector
		),
		MGROUP_TRANSLATE( 
			0,                          // mesh index 
			GRP_BUTTON[2], 1,            // group list and # groups 
			BUTTON_DISTANCE             // tranlation vector
		),
		MGROUP_TRANSLATE( 
			0,                          // mesh index 
			GRP_BUTTON[3], 1,            // group list and # groups 
			BUTTON_DISTANCE              // tranlation vector
		)}; 

	anim_LeftFrontSteer_state = OUTSIDESTEERING_RANGE_F/(OUTSIDESTEERING_RANGE_F+INSIDESTEERING_RANGE_F); 
	anim_RightFrontSteer_state= OUTSIDESTEERING_RANGE_F/(OUTSIDESTEERING_RANGE_F+INSIDESTEERING_RANGE_F);
	anim_LeftRearSteer_state = OUTSIDESTEERING_RANGE_R/(OUTSIDESTEERING_RANGE_R+INSIDESTEERING_RANGE_R); 
	anim_RightRearSteer_state= OUTSIDESTEERING_RANGE_R/(OUTSIDESTEERING_RANGE_R+INSIDESTEERING_RANGE_R);


	anim_LeftFrontSteer = CreateAnimation(anim_LeftFrontSteer_state); 
	anim_RightFrontSteer = CreateAnimation(anim_RightFrontSteer_state);
	anim_LeftRearSteer = CreateAnimation(anim_LeftRearSteer_state); 
	anim_RightRearSteer = CreateAnimation(anim_RightRearSteer_state);

	ANIMATIONCOMPONENT_HANDLE parentFrontLeft = AddAnimationComponent(anim_LeftFrontSteer, 0, 1, &LeftFrontSteer);
	ANIMATIONCOMPONENT_HANDLE parentFrontRight = AddAnimationComponent(anim_RightFrontSteer, 0, 1, &RightFrontSteer);
	ANIMATIONCOMPONENT_HANDLE parentRearLeft = AddAnimationComponent(anim_LeftRearSteer, 0, 1, &LeftRearSteer);
	ANIMATIONCOMPONENT_HANDLE parentRearRight = AddAnimationComponent(anim_RightRearSteer, 0, 1, &RightRearSteer);

	//Wheels 

	//LeftFront
	anim_LeftFront = CreateAnimation(0); 
	AddAnimationComponent(anim_LeftFront, 0, 1, LeftFront,parentFrontLeft); 

	//RightFront
	anim_RightFront = CreateAnimation(0); 
	AddAnimationComponent(anim_RightFront, 0, 1, RightFront,parentFrontRight);

	//LeftRear
	anim_LeftRear = CreateAnimation(0); 
	AddAnimationComponent(anim_LeftRear, 0, 1, LeftRear,parentRearLeft); 

	//RightRear
	anim_RightRear = CreateAnimation(0); 
	AddAnimationComponent(anim_RightRear, 0, 1, RightRear,parentRearRight); 

	//VC
	anim_Speedo = CreateAnimation(0); 
	AddAnimationComponent(anim_Speedo, 0, 1, &Speedo_VC); 

	anim_Rev = CreateAnimation(0); 
	AddAnimationComponent(anim_Rev, 0, 1, &Rev_VC); 

	anim_Fuel = CreateAnimation(0); 
	AddAnimationComponent(anim_Fuel, 0, 1, &Fuel_VC); 

	anim_Gear = CreateAnimation(0.33333); 
	AddAnimationComponent(anim_Gear, 0, 0.33333, &Gear_VC_Left);
	AddAnimationComponent(anim_Gear, 0.33333, 1, &Gear_VC_Right);

	anim_Throttle = CreateAnimation(0);
	AddAnimationComponent(anim_Throttle, 0, 1, &Throttle_VC); 

	anim_Brake = CreateAnimation(0);
	AddAnimationComponent(anim_Brake, 0, 1, &Brake_VC); 

	anim_Steer = CreateAnimation(0.5);
	AddAnimationComponent(anim_Steer, 0, 1, &Steer_VC); 

	anim_Park = CreateAnimation(0);
	AddAnimationComponent(anim_Park, 0, 1, &Park_VC); 

	anim_GearStick = CreateAnimation(0.5); 
	AddAnimationComponent(anim_GearStick, 0, 1, &GearStick_VC);

	anim_Warn = CreateAnimation(0.5);
	AddAnimationComponent(anim_Warn, 0, 1, &Warninglights_VC);

	anim_Head = CreateAnimation(0.5);
	AddAnimationComponent(anim_Head, 0, 1, &Headlights_VC);

	anim_Complex = CreateAnimation(0.5);
	AddAnimationComponent(anim_Complex, 0, 1, &Complex_VC);

	anim_SteerMode = CreateAnimation(0.5);
	AddAnimationComponent(anim_SteerMode, 0, 1, &SteerMode_VC);

	anim_FuelBtn = CreateAnimation(1);
	AddAnimationComponent(anim_FuelBtn, 0, 1,&FuelBtn_VC);

	for(int i = 0; i < 4;i++){
		anim_Button[i] = CreateAnimation(1);
		AddAnimationComponent(anim_Button[i], 0, 1,&Buttons_VC[i]);
	}

	//Init Mesh Pointers
	//visual = NULL;
	vcmesh = NULL;
	vcmesh_dev = NULL;
}


JBT_B1200::~JBT_B1200 ()
{
	//Orbiter
	delete LeftFront;
	delete RightFront;
	delete LeftRear;
	delete RightRear;
	/*
	delete LeftFrontSteerRight;
	delete RightFrontSteerRight;
	delete LeftFrontSteerLeft;
	delete RightFrontSteerLeft;
	*/
	ClearBeacons();
	ClearLightEmitters();
}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Write status to scenario file
// --------------------------------------------------------------
void JBT_B1200::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];
	// default vessel parameters
	VESSEL3::clbkSaveState (scn);
	// Save all UMMU member to scenario
	Crew.SaveAllMembersInOrbiterScenarios(scn);

	// Write custom parameters
	sprintf(cbuf, "%i %lf %i", mode,currentSpeed,parkbrake?1:0);
	oapiWriteScenario_string (scn, "MODE", cbuf);

	sprintf(cbuf, "%i %i %lf", transmission, gear, fuel);
	oapiWriteScenario_string (scn, "TRANS", cbuf);

	sprintf(cbuf, "%lf %lf %lf %lf", lb0, lf1, lb1, lf2);
	oapiWriteScenario_string (scn, "LENG", cbuf);

	sprintf(cbuf, "%lf %lf %lf", theta1, theta2, towbarElevation);
	oapiWriteScenario_string (scn, "ANGL", cbuf);

}

// --------------------------------------------------------------
// Read status from scenario file
// --------------------------------------------------------------
void JBT_B1200::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		// UMMU, Load all saved member from scenario. 
		//if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE) {
			//continue;
		//} else 
		if (!_strnicmp (line,"MODE", 4)) {
			int park=0;
			sscanf (line+4, "%i %lf %i", &mode, &currentSpeed,&park);
			parkbrake = (park == 1);
		} else if (!_strnicmp (line, "TRANS", 5)) {
			sscanf (line+5, "%i %i %lf", &transmission,&gear,&fuel);
		} else if (!_strnicmp (line, "LENG", 4)) {
			sscanf (line+4, "%lf %lf %lf %lf", &lb0, &lf1, &lb1, &lf2);

		} else if (!_strnicmp (line, "ANGL", 4)) {
			sscanf (line+4, "%lf %lf %lf", &theta1, &theta2, &towbarElevation);
		} else if (Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE) {
			continue;
		}
		else {
			ParseScenarioLineEx (line, vs); // unrecognised option - pass to Orbiter's generic parser
		}
		
	}
	
}

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void JBT_B1200::clbkSetClassCaps (FILEHANDLE cfg)
{
	

	// physical vessel parameters
	SetSize(B1200_SIZE);

	SetEmptyMass(B1200_EMPTYMASS);
	//SetPMI (B1200_PMI);
	//SetCrossSections (B1200_CS);
	//SetRotDrag (_V(10000000,0,100000000));
	SetTouchdownPoints(B1200_TDP[0], B1200_TDP[1], B1200_TDP[2]);

	
	/*
	SetMaxWheelbrakeForce(B1200_BRAKEFORCE);
	SetSurfaceFrictionCoeff(0.1,0.7);

	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetRotDrag (_V(1.5,1.5,1.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetTrimScale (0.05);
	*/


	//Create Intial Attachment Points
	front = CreateAttachment(false,B1200_TOWBARATTACHFRONT_START_POS,B1200_TOWBARATTACHFRONT_START_DIR,B1200_TOWBARATTACHFRONT_START_ROT,"T",false);
	back = CreateAttachment(false,B1200_TOWBARATTACHBACK_START_POS,B1200_TOWBARATTACHBACK_START_DIR,B1200_TOWBARATTACHBACK_START_ROT,"T",false);
	fronttow = CreateAttachment(false,B1200_TOWBARATTACHFRONT_POS,B1200_TOWBARATTACHFRONT_DIR,B1200_TOWBARATTACHFRONT_ROT,"TOWBAR",false);
	backtow = CreateAttachment(false,B1200_TOWBARATTACHBACK_POS,B1200_TOWBARATTACHBACK_DIR,B1200_TOWBARATTACHBACK_ROT,"TOWBAR",false);

	//Fuel Tank
	fuelDummy = CreatePropellantResource(1,1);
	fuel = B1200_FUELMASS;
	
	//double fuelFrac = fuel/B1200_FUELMASS;
	//SetPropellantMass(fuelDummy,fuelFrac);
	
	lastLatLongSet = false;
	currentSpeed = 0;
	gear = 0;
	clutch = false;
	steerMode = 0;

	//Thruster, used mainly to get level and for fuel consumption
	//th_Input = CreateThruster(_V(0,0,0),_V(0,0,1),0);
	dummy = CreateThruster(_V(0,0,0),_V(0,0,1),0,fuelDummy);
	CreateThrusterGroup(&dummy,1,THGROUP_MAIN);

	SetCameraOffset(_V(-0.65,1.8,3.5));
	SetCameraCatchAngle(5*RAD);
	//SetCameraRotationRange(90*RAD,90*RAD,45*RAD,40*RAD);
	SetCameraRotationRange(2*PI,2*PI,2*PI,2*PI);

	oapiCameraSetAperture(60*RAD);

	//VC mesh
	//UINT vc = AddMesh ("jbt1200_vc");
	//SetMeshVisibilityMode(vc,MESHVIS_ALWAYS); 

	// associate a mesh for the visual
	//UINT body = AddMesh("jbt1200");
	//SetMeshVisibilityMode(body,MESHVIS_ALWAYS); 
	//AddMesh("jbt1200_vc");
	
	SetMeshVisibilityMode (AddMesh (vcmesh = oapiLoadMeshGlobal ("jbt1200_vc")), MESHVIS_ALWAYS);
	SetMeshVisibilityMode (AddMesh (oapiLoadMeshGlobal ("jbt1200")), MESHVIS_ALWAYS);

	//Engine Setup
	transmission = 0;
	mode = 0;
	starter = false;
	//lastHeading = GetYaw();
	

	//Lengths
	lb0 = 0;
	lf1 = 0;
	lb1 = 0; 
	lf2 = 0;

	//Angles
	theta1 = PI;
	theta2 = 0;

	towbarElevation = 0;

	//Beacons
	
	headlights = false;
	warninglights = true;
	
	parklights = false;
	parkbrake = false;
	brakelights = false;
	
	//Warning Light
	static VECTOR3 warnpos[2] ={{0.77,2.42,3.8},{-0.77,2.42,3.8}};
	static VECTOR3 warncol =_V(1,0.6,0.2);

	for (int i = 0; i < 2; i++) {
		warninglights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		warninglights_beac[i].pos = &warnpos[i];
		warninglights_beac[i].col = &warncol;
		warninglights_beac[i].size = 0.2;
		warninglights_beac[i].falloff = 0.4;
		warninglights_beac[i].period = 0.5;
		warninglights_beac[i].duration = 0.15;
		warninglights_beac[i].tofs = 0+0.25*i;
		warninglights_beac[i].active = warninglights;

		AddBeacon(&warninglights_beac[i]);

	}

	//Illum
	static VECTOR3 illumpos[2] ={{1.22,1.23,-3.05},{-1.22,1.23,-3.05}};
	static VECTOR3 illumcol = _V(1,0.1,0.1);
	for(int i = 0; i < 2; i++) {
		illumlights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		illumlights_beac[i].pos = &illumpos[i];
		illumlights_beac[i].col = &illumcol;
		illumlights_beac[i].size = 0.1;
		illumlights_beac[i].falloff = 0.2;
		illumlights_beac[i].period = 0;
		illumlights_beac[i].duration = 0;
		illumlights_beac[i].tofs = 0;
		illumlights_beac[i].active = headlights;
		AddBeacon(&illumlights_beac[i]);
	}

	//Parklight
	static VECTOR3 parkpos[2] ={{1.22,1.23,-3.05},{-1.22,1.23,-3.05}};
	static VECTOR3 parkcol = _V(1,0.1,0.1);
	for(int i = 0; i < 2; i++) {
		parklights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		parklights_beac[i].pos = &parkpos[i];
		parklights_beac[i].col = &parkcol;
		parklights_beac[i].size = 0.15;
		parklights_beac[i].falloff = 0.3;
		parklights_beac[i].period = 0;
		parklights_beac[i].duration = 0;
		parklights_beac[i].tofs = 0;
		parklights_beac[i].active = parklights;
		AddBeacon(&parklights_beac[i]);
	}

	//Brake
	static VECTOR3 brakepos[2] ={{1.22,1.23,-3.05},{-1.22,1.23,-3.05}};
	static VECTOR3 brakecol = _V(1,0.1,0.1);
	for(int i = 0; i < 2; i++) {
		brakelights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		brakelights_beac[i].pos = &brakepos[i];
		brakelights_beac[i].col = &brakecol;
		brakelights_beac[i].size = 0.2;
		brakelights_beac[i].falloff = 0.4;
		brakelights_beac[i].period = 0;
		brakelights_beac[i].duration = 0;
		brakelights_beac[i].tofs = 0;
		brakelights_beac[i].active = false;
		AddBeacon(&brakelights_beac[i]);
	}
	
	//Reverselights
	static VECTOR3 reversepos[2] ={{0.99,1.23,-3.05},{-0.99,1.23,-3.05}};
	static VECTOR3 reversecol = _V(1,1,1);
	for(int i = 0; i < 2; i++) {
		reverselights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		reverselights_beac[i].pos = &reversepos[i];
		reverselights_beac[i].col = &reversecol;
		reverselights_beac[i].size = 0.2;
		reverselights_beac[i].falloff = 0.4;
		reverselights_beac[i].period = 0;
		reverselights_beac[i].duration = 0;
		reverselights_beac[i].tofs = 0;
		reverselights_beac[i].active = false;
		AddBeacon(&reverselights_beac[i]);
	}

	//Headlights
	static VECTOR3 headspos[2] ={{1.1,0.48,4.55},{-1.1,0.48,4.55}};
	static VECTOR3 headscol = _V(1,1,1);
	for (int i = 0; i < 2; i++) {
		headlights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		headlights_beac[i].pos = &headspos[i];
		headlights_beac[i].col = &headscol;
		headlights_beac[i].size = 0.35;
		headlights_beac[i].falloff = 0.4;
		headlights_beac[i].period = 0;
		headlights_beac[i].duration = 0;
		headlights_beac[i].tofs = 0;
		headlights_beac[i].active = headlights;
		AddBeacon(&headlights_beac[i]);
	}
	
	//Indicators
	static VECTOR3 indicatorcol = _V(1,0.4,0.2);
	
	for (int i = 0; i < 4; i++) {
		indicatorlights_beac[i].shape = BEACONSHAPE_DIFFUSE;
		indicatorlights_beac[i].col = &indicatorcol;
		indicatorlights_beac[i].falloff = 0.4;
		indicatorlights_beac[i].period = 1;
		indicatorlights_beac[i].duration = 0.5;
		indicatorlights_beac[i].tofs = 0;
		indicatorlights_beac[i].active = false;
	}

	//Rear
	static VECTOR3 indicatorRearpos[2] ={{0.76,1.23,-3.05},{-0.76,1.23,-3.05}};
	for (int i = 0; i < 2; i++) {
		indicatorlights_beac[i].pos = &indicatorRearpos[i];
		indicatorlights_beac[i].size = 0.2;
		AddBeacon(&indicatorlights_beac[i]);
	}

	//Front
	static VECTOR3 indicatorFrontpos[2] ={{1.1,0.65,4.55},{-1.1,0.65,4.55}};
	for (int i = 2; i < 4; i++) {
		indicatorlights_beac[i].pos = &indicatorFrontpos[i-2];
		indicatorlights_beac[i].size = 0.2;
		AddBeacon(&indicatorlights_beac[i]);
	}

	/*
	//Rear Side
	static VECTOR3 indicatorRearSidepos[2] ={{1.46,0.65,-3.05},{-1.46,0.65,-3.05}};
	for (int i = 4; i < 6; i++) {
		indicatorlights[i].pos = &indicatorRearSidepos[i-4];
		indicatorlights[i].size = 0.12;
		AddBeacon(&indicatorlights[i]);
	}

	//Front Side
	static VECTOR3 indicatorFrontSidepos[2] ={{1.33,0.58,4.45},{-1.33,0.58,4.45}};
	for (int i = 6; i < 8; i++) {
		indicatorlights[i].pos = &indicatorFrontSidepos[i-6];
		indicatorlights[i].size = 0.12;
		AddBeacon(&indicatorlights[i]);
	}
	*/
	//BEACONLIGHTSPEC warninglight, parklights[2], indicatorlights[8], reverselights[2],headlights[2];
	//LightEmitter* headlights;
	

	//Headlights
	static VECTOR3 headpos =_V(0,0.48,4.55);
	static VECTOR3 headdir = _V(0,0,1);
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	headlight = AddSpotLight(headpos,headdir,50,1e-3,0,1e-3,RAD*25, RAD*60, col_white, col_white, col_a);  

	headlight->Activate(headlights);

	{
	/*
	exhaust_tex = oapiRegisterParticleTexture ("smoke2");
	
	
	if (exhaust_tex != NULL)
		exhaustStream = 1;
	else
		exhaustStream = 0;*/
	/*
	exhaust_prt.flags = 0;
	exhaust_prt.srcsize = 0.2;
	exhaust_prt.srcrate = 125;
	exhaust_prt.v0 = 2;
	exhaust_prt.srcspread = 1;
	exhaust_prt.lifetime = 22;
	exhaust_prt.growthrate = 0.2;
	exhaust_prt.atmslowdown = 0.5;
	exhaust_prt.ltype = PARTICLESTREAMSPEC::DIFFUSE;
	exhaust_prt.levelmap = PARTICLESTREAMSPEC::LVL_LIN;
	exhaust_prt.atmsmap = PARTICLESTREAMSPEC::ATM_FLAT;
	// = exhaust_tex;
	exhaust_prt.tex = oapiRegisterParticleTexture ("smoke");
	exhaustStream = 1;
	*/

		/*DWORD  flags 
  streamspec bitflags 
 double  srcsize 
  particle size at creation [m] 
 double  srcrate 
  average particle creation rate [Hz] 
 double  v0 
  emission velocity [m/s] 
 double  srcspread 
  velocity spread during creation 
 double  lifetime 
  average particle lifetime [s] 
 double  growthrate 
  particle growth rate [m/s] 
 double  atmslowdown 
  slowdown rate in atmosphere 
 enum 
PARTICLESTREAMSPEC::LTYPE  ltype 
  Particle lighting method. 
 enum 
PARTICLESTREAMSPEC::LEVELMAP  levelmap 
  Mapping from level to alpha value (particle opacity). 
 double  lmin 
double  lmax 
  min and max levels for level PLIN and PSQRT mapping types 
 enum 
PARTICLESTREAMSPEC::ATMSMAP  atmsmap 
  mapping from atmospheric params to alpha 
 double  amin 
double  amax 
  min and max densities for atms PLIN mapping 
 SURFHANDLE  tex 
  particle texture handle (NULL for default) 
 */
	//AddParticleStream(&exhaust_prt,_V(0,2.5,0),_V(0,1,0),&exhaustStream);
	//exhaustStream = 1;
 }
	//UMMU
	Crew.InitUmmu(GetHandle());// Must always be the first one
	Crew.DefineAirLockShape(TRUE,-4,4,0,3,2,5);// Airlock open
	Crew.SetMembersPosRotOnEVA(_V(-2,1.2,3.5),_V(0,0,0));// position when EVA, 3 meter in front of ship
	Crew.SetMaxSeatAvailableInShip(2);// Max Numbers of UMMU for this ship
	// Add four default members for when the ship is spawned by scenario editor
	Crew.AddCrewMember("Gary Falcon",28,65,74,"Eng");
	iSelectedUmmuMember=0; // Initialize of select member variable

	Crew.DeclareActionArea(0,B1200_TOWBARATTACHBACK_START_POS,5,TRUE,"action_activated.wav","Vessel Attached to the back");
	Crew.DeclareActionArea(1,B1200_TOWBARATTACHFRONT_START_POS,5,TRUE,"action_activated.wav","Vessel Attached to the front");

	// The “Add  Mmu without scenery editor” variable see PDF doc
	cAddUMmuToVessel[0]=0;

	bool towData = false;
	char towItem[8];
	int itemNo = 0;
	char configBuf[128];
	do {
		sprintf(towItem,"TOW%i",itemNo);
		towData = oapiReadItem_string(cfg,towItem,configBuf);
		if(towData){
			//Read in the values
		}
		itemNo++;
	}while(towData);


	}

// --------------------------------------------------------------
// Overloaded clbkPreStep callback functions
// --------------------------------------------------------------

void JBT_B1200::clbkPreStep (double simt, double simdt, double mjd){

	//UMMU Controls
	if(Crew.GetCrewTotalNumber() < 1 && complex > 0) {
		//Disable the ship
		SetADCtrlMode(0);
		transmission = 0;
	}
	else {
		//Enable the ship
		SetADCtrlMode(7);
	}

	if(vcmesh_dev){

		switch(Crew.GetCrewTotalNumber()){
			case 0:
				SetMeshGroupVisible(vcmesh_dev, 24, false);
				SetMeshGroupVisible(vcmesh_dev, 25, false);
				break;
			case 1:
				SetMeshGroupVisible(vcmesh_dev, 24, true);
				SetMeshGroupVisible(vcmesh_dev, 25, false);
				break;
			case 2:
				SetMeshGroupVisible(vcmesh_dev, 24, true);
				SetMeshGroupVisible(vcmesh_dev, 25, true);
		}
	}

	//New Movement and Steering Code
	//Get mass = Base + Fuel + Ballast + Attachment
	double attachmentMass = 0;
	OBJHANDLE child;
	if(complex) {
		switch(mode){
			case -2:child = GetAttachmentStatus(back);
					attachmentMass += oapiGetFuelMass(child) + oapiGetEmptyMass(child);
			case -1:attachmentMass += B1200_TOWBARMASS;
					break;
			case 2:	child = GetAttachmentStatus(front);
					attachmentMass += oapiGetFuelMass(child) + oapiGetEmptyMass(child);
			case 1:	attachmentMass += B1200_TOWBARMASS;
					break;
		}
	}
	double crewMass = 0;
	for(int i = 0; i < Crew.GetCrewTotalNumber(); i++) {
		crewMass += Crew.GetCrewWeightBySlotNumber(i);
	}
	double mass = B1200_EMPTYMASS + crewMass +fuel + attachmentMass;

	//Get Current Pos
	double lat, lon;
	
	vs.version = 2;
	GetStatusEx(&vs);

	if (vs.status == 1) {
		lon = vs.surf_lng;
		lat = vs.surf_lat;
		lastHead = vs.surf_hdg;
	} else if (lastLatLongSet) {
		lon = lastLong;
		lat = lastLat;
		vs.surf_hdg = lastHead;
	
	} else return;

	//Get Throttle Pos, Gear and Engine Revs
	SetPropellantMass(fuelDummy,1);
	double throttleInput = GetThrusterGroupLevel(THGROUP_MAIN);
	double brakeLevel = 0;

	if(parkbrake){
		brakeLevel = 0.5;
	} else {
		brakeLevel = GetWheelbrakeLevel(2);
	}
		

	//double throttleInput =GetThrusterLevel(th_Input);
	double wheelRPM = (currentSpeed * 60) / (PI * (B1200_TYREDIA/2));
	if(transmission == 0){
		engineRPM = 925; //In neutral, so idle
	}
	else{
		//double engineRPM = wheelRPM * gear * transmission;
		engineRPM = wheelRPM * B1200_GEARS[gear] * B1200_DIFF * B1200_MULTIPLIER * transmission;
		//engineRPM = 1000;
	}

	//If revs in the range, change up or down if possible
	if(engineRPM < 1150 && gear > 0){
		gear--;
		engineRPM = wheelRPM * B1200_GEARS[gear] * B1200_DIFF * B1200_MULTIPLIER * transmission;
	}
	else{
		if(engineRPM > 1800 && gear < 5){
			gear++;
			engineRPM = wheelRPM * B1200_GEARS[gear] * B1200_DIFF * B1200_MULTIPLIER * transmission;
		}
	}
		//Change down if < 1150
			//calc new revs
		//Change up if > 1800
			//calc new revs

	//Set the clutch
	//If idle
	if(engineRPM < 925) {
		clutch = false;
		engineRPM = 925;
	}
	else {
		//Limit the engine
		if(engineRPM > 2075) {
			engineRPM = 2075;
		}
		//Clutch is closed
		clutch = true;
	}

	if(!starter){
		engineRPM = 0;
		clutch = false;
	}

	//Set fuel
	//fuel = GetPropellantMass(fuelDummy);

	//Consume Fuel
	if(fuel > 0 && complex > 0){
		fuel -=B1200_FUELCOMSUMP * simdt * engineRPM;
		//double fuelFrac = fuel/B1200_FUELMASS;
		//SetPropellantMass(fuelDummy,fuel);
	}
		
	double newForce = 0;
	//int active = 0;
	//double engineTorque = 8;

	//See if maxed out or clutch in
	if(starter && ((fuel > 0  && complex > 0) || complex == 0 )){
		//Gas + Engine Revs + Torque Map -> Engine Torque -> New Force at wheels
		//double torque = -0.00112465880214*engineRPM*engineRPM + 3.03599224284932*engineRPM - 1135.148;
		double torque = -0.00112465880214*engineRPM*engineRPM + 3.03599224284932*engineRPM - 800;
		//engineTorque = torque;
		
		torque = torque * throttleInput * B1200_GEARS[gear] * B1200_DIFF * B1200_MULTIPLIER * transmission;
		//engineTorque = torque;
		/*
		switch(transmission){
			case 2:	torque = throttleInput * B1200_GEARS[gear] * B1200_DIFF * 2;
					break;
			case 1:
			case -1:torque = throttleInput * B1200_GEARS[gear] * B1200_DIFF * 1;
					break;
		}*/

		newForce = torque /(B1200_TYREDIA/2);

		
	}
	//Else force = 0

	//Get friction and brakes
	VECTOR3 G;
	GetWeightVector(G);
	double friction = 0.01 * mass * (-G.y/B1200_EMPTYMASS ) + brakeLevel * B1200_BRAKEFORCE;
	
	if(engineRPM == 2075) {
		newForce = friction * transmission;
	}

	//moving forward under power
	if(transmission > 0 && currentSpeed > 0) {
		if(engineRPM == 2075) {
			newForce = 0;
		}
		newForce -= friction;
		//active = 1;
		
	}
	else {
		//moving backward under power
		if(transmission < 0  && currentSpeed < 0) {
			if(engineRPM == 2075) {
				newForce = 0;
			}
			newForce += friction;
			//active = 2;
			
		}
		else{
			//Moving forward with no engine power
			if((transmission == 0 || !clutch) && currentSpeed > 0){
			//if(transmission == 0  && currentSpeed > 0.2){
				newForce = -friction;
				//active = 3;
			}
			else {
				//Moving backwards with no engine power
				if((transmission == 0 || !clutch) && currentSpeed < 0){
					newForce = friction;
					//active = 4;
				}
				else {
					//trying to move forward under power
					if(transmission > 0 && currentSpeed == 0) {
						if (newForce > friction) {
							newForce -= friction;
							//active = 5;
						}
						else{
							newForce = 0;
							//active = 6;
						}
		
					}
					else {
						//trying to move backward under power
						if(transmission < 0 && currentSpeed == 0) {
							if (fabs(newForce) > friction) {
								newForce += friction;
								//active = 7;
							}
							else{
								newForce = 0;
								//active = 8;
							}
						}
						else{
							//Not moving with no engine power
							//if((transmission == 0 || !clutch) && currentSpeed == 0){
								newForce = 0;
								//active = 9;
							//}
			
						}
					}
				}
			}
		}
	}


	//Calc Accel
	double accel = 0;
	if(newForce != 0) {
		accel = newForce / mass;
	}
	
	double nextSpeed = currentSpeed + accel * simdt;
	if (nextSpeed > -0.001 && nextSpeed < 0.001 && currentSpeed != 0) {
		currentSpeed = 0;
	} 
	else {
		if(currentSpeed == 0) {
			currentSpeed = nextSpeed;
		}
		else {
			if((nextSpeed<0) == (currentSpeed<0)) {
				currentSpeed = nextSpeed;
			}
			else {
				currentSpeed = 0;
			}
		}
	}
	//currentSpeed = nextSpeed;

	//Steering
	double turningRadius;
	double avel;
	double dheading;

	//Use RCS setting to set the steering mode if there is no steering lock
	if(GetControlSurfaceLevel(AIRCTRL_RUDDER)==0){

		SURFHANDLE surf = oapiGetTextureHandle(vcmesh,5);

		switch(GetAttitudeMode()) {
			case RCS_LIN:	
						if(mode == 0) {
							steerMode = 2;
							oapiBlt(surf,surf,268,469,112,720,70,84);
						}
						break;
			case RCS_ROT:	steerMode = 1;
							oapiBlt(surf,surf,268,469,15,716,70,84);
							break;
			default: steerMode = 0;
					oapiBlt(surf,surf,268,469,208,718,70,84);
		}//switch
	}//if

	if(steerMode == 0) {
		turningRadius = B1200_WHEELBASE / sin(B1200_TURNINGANGLE_2WS * GetControlSurfaceLevel(AIRCTRL_RUDDER));
		avel= currentSpeed/turningRadius;

		//Get new heading
		dheading = avel * simdt;
		vs.surf_hdg += dheading;
		if(vs.surf_hdg< 0) vs.surf_hdg += 2.0 * PI;
		if(vs.surf_hdg >= 2.0 * PI) vs.surf_hdg -= 2.0 * PI;
	}
	if(steerMode == 1) {
		turningRadius = B1200_WHEELBASE / sin(B1200_TURNINGANGLE_4WS * GetControlSurfaceLevel(AIRCTRL_RUDDER));
		avel= currentSpeed/turningRadius;

		//Get new heading
		dheading = avel * simdt;
		vs.surf_hdg += dheading;
		if(vs.surf_hdg< 0) vs.surf_hdg += 2.0 * PI;
		if(vs.surf_hdg >= 2.0 * PI) vs.surf_hdg -= 2.0 * PI;
	}
	if(steerMode == 2) {
		//turningRadius = B1200_WHEELBASE / sin(B1200_TURNINGANGLE_CRAB * GetControlSurfaceLevel(AIRCTRL_RUDDER));
		//avel= currentSpeed/turningRadius;

		//Get new heading
		dheading = B1200_TURNINGANGLE_CRAB * GetControlSurfaceLevel(AIRCTRL_RUDDER);
		avel = 0;
	}
	//Calc new Position (Crawer.cpp SSU)
	VECTOR3 dir = _V(sin(lastHead+dheading), cos(lat) * cos(lastHead+dheading), 0);
	dir /= length(dir);
	hEarth = GetGravityRef();
	lon += dir.x * currentSpeed * simdt / oapiGetSize(hEarth);
	lat += dir.y * currentSpeed * simdt / oapiGetSize(hEarth);
	vs.surf_lng = lon;
	vs.surf_lat = lat;
	vs.status = 1;
	DefSetStateEx(&vs);

	//Store the current set for next time.
	lastLat = lat;
	lastLong = lon;
	lastHead = vs.surf_hdg;
	lastLatLongSet = true;

	//Towing code
	//Angular Speeds
	double omega0 = avel,omega1 = 0,omega2 = 0;
	//Velocities
	double v0 = currentSpeed,v1 = 0,v2 = 0;

	if(mode == -1){
		v1 = v0*cos(theta1)-omega0*lb0*sin(theta1);
		omega1 = (v0*sin(theta1)-omega0*lb0*cos(theta1))/lf1;

		theta1 = theta1 + (omega1-omega0) * simdt;

		//theta1 +=PI;

		//if out of bounds, stop, cut engine, netural
		if(theta1-PI > PI/2 || theta1-PI < -PI/2){
			currentSpeed = 0;
			starter = false;
			transmission = 0;
			theta1 =PI + (theta1-PI) > 0?PI/2:-PI/2;
			PlayVesselWave(MyID,CLANG,NOLOOP);
			PlayVesselWave(MyID,SCREACH,NOLOOP);
		}

		ChangeAttachment(backtow,towbarElevation,theta1,-1);
		
	}
	if(mode == -2){
		v1 = v0*cos(theta1)-omega0*lb0*sin(theta1);
		omega1 = (v0*sin(theta1)-omega0*lb0*cos(theta1))/lf1;

		theta1 = theta1 + (omega1-omega0) * simdt;

		//theta1 +=PI;

		//if out of bounds, stop, cut engine, netural
		if(theta1-PI > PI/2 || theta1-PI < -PI/2){
			currentSpeed = 0;
			starter = false;
			transmission = 0;
			theta1 =PI + (theta1-PI) > 0?PI/2:-PI/2;
			PlayVesselWave(MyID,CLANG,NOLOOP);
			PlayVesselWave(MyID,SCREACH,NOLOOP);
		}

		ChangeAttachment(backtow,towbarElevation,theta1,-2);

		v2 = v1*cos(theta2)-omega1*lb1*sin(theta2);
		omega2 = (v1*sin(theta2)-omega1*lb1*cos(theta2))/lf2;

		theta2 = theta2 + (omega2-omega1-omega0) * simdt;
		//theta2 +=PI;

		ChangeAttachment(back,0,theta2,0);

	}
	if(mode == 1){
		v1 = v0*cos(theta1)-omega0*lb0*sin(theta1);
		omega1 = (v0*sin(theta1)-omega0*lb0*cos(theta1))/lf1;

		theta1 = theta1 + (omega1-omega0) * simdt;

		//if out of bounds, stop, cut engine, netural
		if(theta1 > PI/2 || theta1 < -PI/2){
			currentSpeed = 0;
			starter = false;
			transmission = 0;
			theta1 = theta1 > 0?PI/2:-PI/2;
			PlayVesselWave(MyID,CLANG,NOLOOP);
			PlayVesselWave(MyID,SCREACH,NOLOOP);
		}

		ChangeAttachment(fronttow,towbarElevation,theta1,1);

	}

	if(mode == 2){
		v1 = v0*cos(theta1)-omega0*lb0*sin(theta1);
		omega1 = (v0*sin(theta1)-omega0*lb0*cos(theta1))/lf1;

		v2 = v1*cos(theta2)-omega1*lb1*sin(theta2);
		omega2 = (v1*sin(theta2)-omega1*lb1*cos(theta2))/lf2;

		theta1 = theta1 + (omega1-omega0) * simdt;

		//if out of bounds, stop, cut engine, netural
		if(theta1 > PI/2 || theta1 < -PI/2){
			currentSpeed = 0;
			starter = false;
			transmission = 0;
			theta1 = theta1 > 0?PI/2:-PI/2;
			PlayVesselWave(MyID,CLANG,NOLOOP);
			PlayVesselWave(MyID,SCREACH,NOLOOP);
		}

		ChangeAttachment(fronttow,towbarElevation,theta1,2);

		theta2 = theta2 + (omega2-omega1-omega0) * simdt;
		ChangeAttachment(front,0,theta2,0);

	}

	//sprintf (oapiDebugString(), "Fuel: %f Revs: %f Force: %f In: %f Steer:%i",fuel,engineRPM,newForce,throttleInput,steerMode);
	//sprintf (oapiDebugString(), "lb0: %f lf1: %f lb1: %f lf2: %f towingElevation: %f",lb0,lf1,lb1,lf2,towbarElevation);


	//Set sounds
	if(starter) {
		int freq = (int)(22.6154 * engineRPM);
		
		PlayVesselWave(MyID, LOW,LOOP,255,freq);
	}
	else {
		StopVesselWave(MyID,LOW);
	}
	
	if(currentSpeed != 0) {
		int vol = (int)(fabs(currentSpeed)/6.0*255);
		PlayVesselWave(MyID,GROUND,LOOP,vol);
	}
	else {
		StopVesselWave(MyID,GROUND);
	}

	//Set Animations

	//Wheel Speed
	double rotationalSpeed = currentSpeed/(B1200_TYREDIA/2);
	double rotStep = rotationalSpeed * simdt;
	double fracStep = rotStep / (2*PI);
	
	//Left Front
	anim_LeftFront_state += fracStep;

	if(anim_LeftFront_state > 1)
		anim_LeftFront_state -= 1;
	if(anim_LeftFront_state < 0)
		anim_LeftFront_state += 1;
	//anim_LeftFront_state = 0;
	SetAnimation(anim_LeftFront,anim_LeftFront_state);

	//Right Front
	anim_RightFront_state += fracStep;

	if(anim_RightFront_state > 1)
		anim_RightFront_state -= 1;
	if(anim_RightFront_state < 0)
		anim_RightFront_state += 1;
	//anim_RightFront_state = 0;
	SetAnimation(anim_RightFront,anim_RightFront_state);

	//Left Rear
	anim_LeftRear_state += fracStep;

	if(anim_LeftRear_state > 1)
		anim_LeftRear_state -= 1;
	if(anim_LeftRear_state < 0)
		anim_LeftRear_state += 1;

	SetAnimation(anim_LeftRear,anim_LeftRear_state);

	//Right Rear
	anim_RightRear_state += fracStep;

	if(anim_RightRear_state > 1)
		anim_RightRear_state -= 1;
	if(anim_RightRear_state < 0)
		anim_RightRear_state += 1;

	SetAnimation(anim_RightRear,anim_RightRear_state);

	//Light Control
	double turn = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	//If it is in the last 50% of the second
	bool blink = (simt-(int)simt) > 0.5;
	if(turn > 0) {
		//Turn on Right side
		for (int i = 0; i < 4; i += 2) {
			indicatorlights_beac[i].active = true;	
		}
		//Turn off Left side
		for (int i = 1; i < 4; i += 2) {
			indicatorlights_beac[i].active = false;
		}
		if(vcmesh_dev)
			{
				if(blink)
				{
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.8,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,5,&illum); 
					if(!IsPlaying(MyID,BLINK)){
						PlayVesselWave(MyID,BLINK,NOLOOP); 

					}
				}
				else {
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.1,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,5,&illum); 
				}
				{
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.1,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,2,&illum); 
				}
			}

	}//if turn
	if(turn < 0) {
		//Turn on Right side
		for (int i = 0; i < 4; i += 2) {
			indicatorlights_beac[i].active = false;
		}
		//Turn off Left side
		for (int i = 1; i < 4; i += 2) {
			indicatorlights_beac[i].active = true;
		}
		if(vcmesh_dev)
			{
				
				if(blink)
				{
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.8,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,2,&illum); 
					if(!IsPlaying(MyID,BLINK)){
						PlayVesselWave(MyID,BLINK,NOLOOP); 
					}
				}
				else {
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.1,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,2,&illum); 
				}
				{
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.1,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,5,&illum); 
				}
			}
	}//if turn
	if(turn == 0) {
		//Turn off all
		for (int i = 0; i < 4; i ++) {
			indicatorlights_beac[i].active = false;
		}
		if(vcmesh_dev)
			{
				{
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.1,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,5,&illum); 
				}
				{
					COLOUR4 d = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 a = {0.168627,0.588235,0.00392157,1.0};
					COLOUR4 s = {0.9,0.9,0.9,1.0};
					COLOUR4 e = {0.0,0.1,0.0,1.0};
					MATERIAL illum = {d,a,s,e,40};
					oapiSetMaterial(vcmesh_dev,2,&illum); 
				}
			}

	}//if turn

	//double brakelevel = GetWheelbrakeLevel(1);
	if (parkbrake) {
		parklights = false;
		parkLights();
	}
	else {
		parklights = true;
		parkLights();
		if(brakeLevel > 0) {
			brakelights = false;
			brakeLights();
		}
		else {
			brakelights = true;
			brakeLights();
		}

	}//else

	if (transmission == -1) {
		reverselights_beac[0].active = true;
		reverselights_beac[1].active = true;
	}
	else {
		reverselights_beac[0].active = false;
		reverselights_beac[1].active = false;
	}

	//2WS or 4WS
	if(steerMode == 0 || steerMode == 1) {
		double centrefrac = B1200_TURNINGANGLEOUTSIDE_F/(B1200_TURNINGANGLEOUTSIDE_F+B1200_TURNINGANGLEINSIDE_F);

		anim_LeftFrontSteer_state = centrefrac; 
		anim_RightFrontSteer_state= centrefrac;
		if(turn == 0) {
			anim_LeftFrontSteer_state = centrefrac; 
			anim_RightFrontSteer_state= centrefrac;
		}
		else {
			if (turn > 0){
				//anim_LeftFrontSteer_state = (1-centrefrac)*turn+centrefrac;
				anim_RightFrontSteer_state= (1-centrefrac)*turn+centrefrac;
				anim_LeftFrontSteer_state = centrefrac*(1-turn);
				//anim_RightFrontSteer_state=0;
				//anim_LeftFrontSteer_state=1;

			}//if positive
			else {
				//
				anim_LeftFrontSteer_state = (1-centrefrac)*-turn+centrefrac;
				anim_RightFrontSteer_state= centrefrac*(turn+1);

			}//else negative
		}
		SetAnimation(anim_LeftFrontSteer,anim_LeftFrontSteer_state);
		SetAnimation(anim_RightFrontSteer,anim_RightFrontSteer_state);
	}

	//4WS
	if(steerMode == 1) {
		double centrefrac = B1200_TURNINGANGLEOUTSIDE_R/(B1200_TURNINGANGLEOUTSIDE_R+B1200_TURNINGANGLEINSIDE_R);

		anim_LeftRearSteer_state = centrefrac; 
		anim_RightRearSteer_state= centrefrac;
		if(turn == 0) {
			anim_LeftRearSteer_state = centrefrac; 
			anim_RightRearSteer_state= centrefrac;
		}
		else {
			if (turn > 0){
				anim_LeftRearSteer_state = centrefrac*(1-turn);

				anim_RightRearSteer_state= (1-centrefrac)*turn+centrefrac;
	
			}//if positive
			else {
				//
				anim_LeftRearSteer_state = (1-centrefrac)*-turn+centrefrac;
				anim_RightRearSteer_state= centrefrac*(turn+1);

			}//else negative
		}
		SetAnimation(anim_LeftRearSteer,anim_LeftRearSteer_state);
		SetAnimation(anim_RightRearSteer,anim_RightRearSteer_state);
	}
	else{
		double centrefrac = B1200_TURNINGANGLEOUTSIDE_R/(B1200_TURNINGANGLEOUTSIDE_R+B1200_TURNINGANGLEINSIDE_R);

		anim_LeftRearSteer_state = centrefrac; 
		anim_RightRearSteer_state= centrefrac;
		SetAnimation(anim_LeftRearSteer,anim_LeftRearSteer_state);
		SetAnimation(anim_RightRearSteer,anim_RightRearSteer_state);
	}

	//Crab
	if(steerMode == 2) {
		double centrefracR = B1200_TURNINGANGLEOUTSIDE_R/(B1200_TURNINGANGLEOUTSIDE_R+B1200_TURNINGANGLEINSIDE_R);
		double centrefracF = B1200_TURNINGANGLEOUTSIDE_F/(B1200_TURNINGANGLEOUTSIDE_F+B1200_TURNINGANGLEINSIDE_F);
		double crabFracR = B1200_TURNINGANGLE_CRAB/(B1200_TURNINGANGLEOUTSIDE_R+B1200_TURNINGANGLEINSIDE_R);
		double crabFracF = B1200_TURNINGANGLE_CRAB/(B1200_TURNINGANGLEOUTSIDE_F+B1200_TURNINGANGLEINSIDE_F);
		

		if(turn == 0) {
			anim_LeftRearSteer_state = centrefracR; 
			anim_RightRearSteer_state= centrefracR;
			anim_LeftFrontSteer_state = centrefracF; 
			anim_RightFrontSteer_state= centrefracF;

		}
		else {
			if (turn > 0){
				anim_LeftRearSteer_state = centrefracR+crabFracR*turn; 
				anim_RightRearSteer_state= centrefracR-crabFracR*turn;
				anim_LeftFrontSteer_state = centrefracF-crabFracF*turn; 
				anim_RightFrontSteer_state= centrefracF+crabFracF*turn;
	
			}//if positive
			else {
				//
				anim_LeftRearSteer_state = centrefracR-crabFracR*-turn; 
				anim_RightRearSteer_state= centrefracR+crabFracR*-turn;
				anim_LeftFrontSteer_state = centrefracF+crabFracF*-turn; 
				anim_RightFrontSteer_state= centrefracF-crabFracF*-turn;
			}//else negative
		}

		SetAnimation(anim_LeftFrontSteer,anim_LeftFrontSteer_state);
		SetAnimation(anim_RightFrontSteer,anim_RightFrontSteer_state);
		SetAnimation(anim_LeftRearSteer,anim_LeftRearSteer_state);
		SetAnimation(anim_RightRearSteer,anim_RightRearSteer_state);
	}

	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
		{
		case UMMU_TRANSFERED_TO_OUR_SHIP: 
											sprintf(SendHudMessage(),"%s \"%s\" aged %i was transfered to our ship",
											Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
											,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
											break;
		case UMMU_RETURNED_TO_OUR_SHIP:
											sprintf(SendHudMessage(),"%s \"%s\" aged %i entered into our ship",
											Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
											Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}

	int ActionAreaReturnCode=Crew.DetectActionAreaActivated();
	if(ActionAreaReturnCode>-1)
	{
		// this is just an example, we have four area declared
		// action area ID 0 triggered 
		if(ActionAreaReturnCode==0)
		{
			attachAction = true;
			if(mode == -2) {
				Crew.SetActionAreaText(0,"Vessel Dettached from the back");	
				char kstate[256];
				ZeroMemory(kstate, sizeof(kstate) );
				//kstate[OAPI_KEY_A] = (char)0x80;
				kstate[OAPI_KEY_LSHIFT] = (char)0x80;
				SendBufferedKey(OAPI_KEY_A,true,kstate);
				
			}
			else {
				Crew.SetActionAreaText(0,"Vessel Attached to the back");	
				SendBufferedKey(OAPI_KEY_A);

			}
		}
		// action area ID 1 triggered
		else if(ActionAreaReturnCode==1)
		{
			attachAction = true;
			if(mode == 2) {
				Crew.SetActionAreaText(0,"Vessel Dettached from the front");
				char kstate[256];
				ZeroMemory(kstate, sizeof(kstate) );
				kstate[OAPI_KEY_D] = (char)0x80;
				kstate[OAPI_KEY_LSHIFT] = (char)0x80;
				SendBufferedKey(OAPI_KEY_D,true,kstate);
			}
			else {
				Crew.SetActionAreaText(0,"Vessel Attached to the front");	
				SendBufferedKey(OAPI_KEY_D);
			}
		}
			
	}

	// "AddUMmuToVessel" function
	AddUMmuToVessel();

	Crew.WarnUserUMMUNotInstalled("JBT B1200"); 

	if(currentSpeed == 0) {
		timeIdle += simdt;
		if(timeIdle > 300){
			starter = false;
		}
	}
	else {
		timeIdle = 0;
	}

}//Prestep

// --------------------------------------------------------------
// Overloaded clbkConsumeBufferedKey callback functions
// --------------------------------------------------------------

int JBT_B1200::clbkConsumeBufferedKey(DWORD key,bool  down,char *  kstate ) 
 {
	// only process keydown events
	if (!down) 
		return 1; 
	 
	  if (key == OAPI_KEY_W) {
		if(transmission < 2  && currentSpeed == 0)
			transmission++;
			oapiVCTriggerRedrawArea (0,VC_GEAR);
			oapiVCTriggerRedrawArea (1,VC_GEAR);
		}
	if (key == OAPI_KEY_S) {
		if(transmission > -1 && currentSpeed == 0)
			transmission--;
			oapiVCTriggerRedrawArea (0,VC_GEAR);
			oapiVCTriggerRedrawArea (1,VC_GEAR);
		}

	if (key == OAPI_KEY_V) {
		if(AttachTowbar(true,true)){
			mode = 1;
			PlayVesselWave(MyID,CLANG,NOLOOP);
		}
	}
	if (key == OAPI_KEY_B) {
		if(AttachTowbar(false,true)){
			mode = -1;
			PlayVesselWave(MyID,CLANG,NOLOOP);
		}
	}

	if (key == OAPI_KEY_A) {
		if (complex == 0) {
			if(mode == 0) {
				AttachTowbar(false,true);
				mode = -1;
			}//if mode == 0
			if(mode == -1 && AttachPlane(false,true)){
				mode = -2;
				PlayVesselWave(MyID,CLANG,NOLOOP);
			}
		}//complex
		else {
			if (attachAction){
				if(mode == 0) {
					AttachTowbar(false,true);
					mode = -1;
				}//if mode == 0
				if(mode == -1 && AttachPlane(false,true)){
					mode = -2;
					PlayVesselWave(MyID,CLANG,NOLOOP);
				}
				attachAction = false;
			}//attach
		}//complex
	}//if A

	if (key == OAPI_KEY_D) {
		if (complex == 0) {
			if(mode == 0) {
				AttachTowbar(true,true);
				mode = 1;
			}//if mode == 0
			if(mode == 1 && AttachPlane(true,true)){
				mode = 2;
				PlayVesselWave(MyID,CLANG,NOLOOP);
			}

		}//complex
		else {
			if (attachAction){
				if(mode == 0) {
					AttachTowbar(true,true);
					mode = 1;
				}//if mode == 0
				if(mode == 1 && AttachPlane(true,true)){
					mode = 2;
					PlayVesselWave(MyID,CLANG,NOLOOP);
				}
				attachAction = false;
			}//attach
		}//complex
	}//if D

	if (KEYDOWN (kstate, OAPI_KEY_LSHIFT)) {
		if (key == OAPI_KEY_B) {
			if(AttachTowbar(false,false) && (mode == -1 || mode == 1)){
				mode = 0;
				PlayVesselWave(MyID,CLANG,NOLOOP);
			}
		}//Key B
		if (key == OAPI_KEY_A) {
			if (complex == 0) {
				if(AttachPlane(false,false)){
					mode = -1;
					PlayVesselWave(MyID,CLANG,NOLOOP);
				}
			}//complex
			else {
				if (attachAction){
					if(AttachPlane(false,false)){
						mode = -1;
						PlayVesselWave(MyID,CLANG,NOLOOP);
					}
					attachAction = false;
				}//attach
			}//complex
		}//if A

		if (key == OAPI_KEY_D) {
			if (complex == 0) {
				if(AttachPlane(true,false)){
					mode = 1;
					PlayVesselWave(MyID,CLANG,NOLOOP);
				}
			}//complex
			else {
				if (attachAction){
					if(AttachPlane(true,false)){
						mode = 1;
						PlayVesselWave(MyID,CLANG,NOLOOP);
					}
						attachAction = false;
				}//attach
			}//complex
		}//if D
	}//if Shift
	
	//Indicators
	if (KEYDOWN (kstate, OAPI_KEY_LSHIFT)) {
		if (key == OAPI_KEY_H) {
			headlights = headLights();
			oapiVCTriggerRedrawArea (0,VC_HEADLIGHT);
		}
		if (key == OAPI_KEY_F) {
			warninglights = warningLights();
			oapiVCTriggerRedrawArea (0,VC_WARNINGLIGHT);
		}
		if (key == OAPI_KEY_P) {
			//Park Brake
			parkbrake = !parkbrake;
			oapiVCTriggerRedrawArea (0,VC_PARK);
			oapiVCTriggerRedrawArea (1,VC_PARK);
		}
	}//LShift

	if(KEYDOWN (kstate, OAPI_KEY_LCONTROL) && (KEYDOWN (kstate, OAPI_KEY_O))) {
		//Flip the complex flight model var	
		complex ^=1;
		oapiVCTriggerRedrawArea (0,VC_COMPLEX);
	}
	if(KEYDOWN (kstate, OAPI_KEY_LCONTROL) && (KEYDOWN (kstate, OAPI_KEY_E))) {
		starter = !starter;
		timeIdle = 0;
		if(starter)
			PlayVesselWave(MyID,STARTER,NOLOOP);
	 }
	if(KEYDOWN (kstate, OAPI_KEY_LCONTROL) && (KEYDOWN (kstate, OAPI_KEY_F)) && currentSpeed == 0) {
		fuel = B1200_FUELMASS;
	 }

	//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		if(currentSpeed == 0){
			Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(iSelectedUmmuMember));
		}

		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			return 1;
		}

		// we test that we select existing member
		if(iSelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			iSelectedUmmuMember++;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			return 1;
		}
		if(iSelectedUmmuMember>0)
			iSelectedUmmuMember--;

		return 1;
	}


	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	if(key==OAPI_KEY_4&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		AddUMmuToVessel(TRUE);
	}


	return 0;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{

	return new JBT_B1200 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	//Orbiter

	if (vessel) delete (JBT_B1200*)vessel;
}

// --------------------------------------------------------------
// Respond to generic messages
// --------------------------------------------------------------
int JBT_B1200::clbkGeneric (int msgid, int prm, void *context)
{
	switch (msgid) {
	case VMSG_LUAINTERPRETER:
		return Lua_InitInterpreter (context);
	case VMSG_LUAINSTANCE:
		return Lua_InitInstance (context);
	}
	return 0;
}

// --------------------------------------------------------------
// Overloaded clbkPostCreation callback functions
// --------------------------------------------------------------
void JBT_B1200::clbkPostCreation (void)
{

	////////////////////////////////////////////////////////////////
	// 3-ORBITERSOUND EXAMPLE - INIT AND LOADING OF WAV
	// THIS MUST BE CALLED ABSOLUTELY IN THE "POSTCREATION CALLBACK" 
	////////////////////////////////////////////////////////////////
	// here we connect to OrbiterSound and store the returned ID in your class
	// this is the first thing to do. You must call this in "clbkPostCreation" 
	// (new version of ovcPostCreation wich is now obsolet)
	
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	SetMyDefaultWaveDirectory("Sound\\_CustomVesselsSounds\\JBT_B1200");

	//Load the sound files
	RequestLoadVesselWave(MyID,GROUND,"groundspeed.wav",INTERNAL_ONLY);
	//RequestLoadVesselWave(MyID,PARKBRAKES,"parkbrake.wav",INTERNAL_ONLY);
	RequestLoadVesselWave(MyID,CLANG,"clang.wav",BOTHVIEW_FADED_CLOSE);
	RequestLoadVesselWave(MyID,SCREACH,"screach.wav",BOTHVIEW_FADED_CLOSE);
	RequestLoadVesselWave(MyID,BLINK,"blinker.wav",INTERNAL_ONLY);

	//RequestLoad3DWaveMono(MyID,IDLE,"Sound\\JBT_B1200\\idle.wav",BOTHVIEW_FADED_CLOSE,&_V(0,1,-1));

	// Now we'll use the 'Set3dWavParameters' on the loaded wave.
	// This function allows also to change range and position 
	// but we'll use it to set only the directional cone now:
	OS3DCONE MyCone={0};				// init all struct value to zero
	MyCone.dwInsideConeAngle=90;		// the cone angle where the sound is at max (255)
	MyCone.dwOutsideConeAngle=360;		// the cone angle where the sound is lower (progressive).
	MyCone.vConeOrientation=_V(0,0,0-1);	// normalized vector, orientation of cone, here left.
	MyCone.lConeOutsideVolume=240;		// outside cone volume ( max 255), usually a 230 value 
										// is more realistic, but we want a stunning effect

	float B1200_MINSOUND = 10;
	float B1200_MAXSOUND = 400;

	RequestLoad3DWaveMono(MyID,LOW,"low.wav",BOTHVIEW_FADED_CLOSE,&_V(0,1,-1));
	Set3dWaveParameters(MyID,LOW,&_V(0,1,-1),NULL,&B1200_MINSOUND,&B1200_MAXSOUND,&MyCone);
	RequestLoad3DWaveMono(MyID,STARTER,"starter.wav",BOTHVIEW_FADED_CLOSE,&_V(0,1,-1));
	Set3dWaveParameters(MyID,STARTER,&_V(0,1,-1),NULL,&B1200_MINSOUND,&B1200_MAXSOUND,&MyCone);

	
//Anim Set up
	anim_LeftFront_state = 0;
	anim_RightFront_state = 0;
	anim_LeftRear_state = 0;
	anim_RightRear_state = 0;

	//Trigger all redraws
	
	timeIdle = 0;
}


// ==============================================================
// API callback interface
// ==============================================================
// ==============================================================
// User defined functions
// ==============================================================

//Returns true if attach is successful
//Returns false if attach unsucessful or detached child
bool JBT_B1200::AttachTowbar (bool frontPoint, bool Attach){
	//Check for attach

	if(!Attach){
		//Deattach front and rear
		//Return if one is true.
		DetachChild(fronttow);
		DetachChild(backtow);
		return true;
	}
	else{
	//If true
		//look for vessel_Name_towbar
		char towName[256] = "";
		char* vesselName;
		vesselName = GetName();
		strcat(towName, vesselName);
		strcat(towName,"_Towbar");
		OBJHANDLE childVessel = oapiGetVesselByName(towName);
		//if NULL
		if (childVessel == NULL){
			VESSELSTATUS vs;
			
			GetStatus(vs);
			childVessel = oapiCreateVessel(towName,"Towbar",vs);
		}
			//Create towbar vessel
			//If failed, return false
		//Check rear or front attach
		if (childVessel != NULL){
			//set the pointer to the candidate vessel
			VESSEL *v = oapiGetVesselInterface (childVessel);
			//Find out the number of child attachment points
			DWORD nAttach = v->AttachmentCount (true);

			for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate

				//Get the attachment handle of the candidate vessels child attachment point
				ATTACHMENTHANDLE childAttachHandle = v->GetAttachmentHandle (true, j);

				//Load the ID
				const char *id = v->GetAttachmentId (childAttachHandle);
					
				//Check then ID
				if (strncmp (id, "TOWBAR", 6)) continue; // attachment point not compatible

				//Front or back attachment point
				if(frontPoint){
					if (AttachChild (childVessel, fronttow, childAttachHandle)) {
						//ChangeAttachment(fronttow,B1200_TOWBARATTACHFRONT_POS,TOWBARANGLE,0);
						
						//Lengths
						lb0 = B1200_TOWBARATTACHFRONT_POS.z;
						lf1 = B1200_TOWBARLENGTH/2;

						//Angles
						theta1 = 0;

						towbarElevation = TOWBARANGLE;

						return true;
					}//if attach
					else {
						return false;
					}//else attach
				}//frontPoint
				else {

					if (AttachChild (childVessel, backtow, childAttachHandle)) {
						//ChangeAttachment(fronttow,B1200_TOWBARATTACHFRONT_POS,TOWBARANGLE,0);
						
						//Lengths
						lb0 = B1200_TOWBARATTACHBACK_POS.z;
						lf1 = B1200_TOWBARLENGTH/2;

						//Angles
						theta1 = PI;

						towbarElevation = TOWBARANGLE;
						return true;
					}//if attach
					else {
						return false;
					}//else attach
				}//else
			}//for scan
		}//If vessel not null
	}//Else 
		
return false;
}

//Returns true if attach is successful
//Returns false if attach unsucessful or detached child
bool JBT_B1200::AttachPlane (bool frontPoint, bool Attach) {
	//Only use if we are detaching
	if(!Attach) {
		//If a vessel is attached, detach it
		if (frontPoint) {
			if(GetAttachmentStatus(front) != NULL){
				SetEmptyMass (B1200_EMPTYMASS + B1200_TOWBARMASS);
				DetachChild(front);
				//Lengths
				lb0 = B1200_TOWBARATTACHFRONT_POS.z;
				lf1 = B1200_TOWBARLENGTH/2;
				//Angles
				towbarElevation = TOWBARANGLE;
				return true;
				}//if
		}//if

		else {
			if(GetAttachmentStatus(back) != NULL){
				SetEmptyMass (B1200_EMPTYMASS + B1200_TOWBARMASS);

				//Lengths
				lb0 = B1200_TOWBARATTACHBACK_POS.z;
				lf1 = B1200_TOWBARLENGTH/2;

				//Angles
				towbarElevation = TOWBARANGLE;

				DetachChild(back);
				return true;
				}//if
		}//else
	}//If attach
	
	//Only do if we are attaching
	else {
		VECTOR3 childGlobalPos,childAttachGlobalPos, AttachFrontGlobalPos,frontPos, AttachBackGlobalPos, backPos, childAttachpos, dir, rot; //dir and rot not used
		//Get the global position of the parent attachment points 
		GetAttachmentParams (front, frontPos, dir, rot);
		GetAttachmentParams (back, backPos, dir, rot);

		Local2Global (frontPos, AttachFrontGlobalPos);
		Local2Global (backPos, AttachBackGlobalPos);
		
		// Search the complete vessel list for a grappling candidate.
		// Not very scalable ...
		for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
			
			
			//Load the object handle of the current target
			OBJHANDLE childVessel = oapiGetVesselByIndex (i);

			if (childVessel == GetHandle()) continue; // we don't want to grapple ourselves ...

			//Get the global position of the target vessel
			oapiGetGlobalPos (childVessel, &childGlobalPos);
			
			//Only target vessels which have the parent attachment point inside their size
			if (dist (childGlobalPos, AttachFrontGlobalPos) < (10*oapiGetSize (childVessel))) { // in range
				
				//set the pointer to the candidate vessel
				VESSEL *v = oapiGetVesselInterface (childVessel);
				
				//Find out the number of child attachment points
				DWORD nAttach = v->AttachmentCount (true);

				for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate

					//Get the attachment handle of the candidate vessels child attachment point
					ATTACHMENTHANDLE childAttachHandle = v->GetAttachmentHandle (true, j);

					//Load the ID
					const char *id = v->GetAttachmentId (childAttachHandle);
					
					//Check then ID
					if (strncmp (id, "T", 2)) continue; // attachment point not compatible
					
					//Get the attachment parameters
					v->GetAttachmentParams (childAttachHandle, childAttachpos, dir, rot);
					
					//Translate the local position to global a one
					v->Local2Global (childAttachpos, childAttachGlobalPos);
					
					//Front or back attachment point
					if(frontPoint){
						//See if the attachment is in range
						if (dist (childAttachGlobalPos, AttachFrontGlobalPos) < MAX_GRAPPLING_DIST) { // found one!
							if(AttachChild (childVessel, front, childAttachHandle)) {
								
								VECTOR3 touchdown0,touchdown1,touchdown2;
								v->GetTouchdownPoints(touchdown0,touchdown1,touchdown2);

								double heightDiff = B1200_TOWBARATTACHFRONT_POS.y - B1200_COG - (childAttachpos.y-touchdown0.y) - 0.001;

								towbarElevation = -asin(heightDiff/B1200_TOWBARLENGTH);
								//towbarElevation = heightDiff;
								double towbarLength = B1200_TOWBARLENGTH*cos(towbarElevation);
								
								//Lengths
								lb0 = B1200_TOWBARATTACHFRONT_POS.z;
								lf1 = towbarLength + (childAttachpos.z-touchdown0.z);
								lb1 = 0.05;
								lf2 = childAttachpos.z-(touchdown1.z+touchdown2.z)/2;

								//Angles
								theta2 = 0;

								return true;
							}//if attach
							else {
								return false;
							}//else attach
						}//if
					}//frontPoint
					else {
						//See if the attachment is in range
						if (dist (childAttachGlobalPos, AttachBackGlobalPos) < MAX_GRAPPLING_DIST) { // found one!

							//SetEmptyMass (B1200_EMPTYMASS + B1200_TOWBARMASS + m2);
							if(AttachChild (childVessel, back, childAttachHandle)){
								

								//Setup the towbar
								char * classname = v->GetClassName();

								for(unsigned int i = 0;i < towbarData.size();i++){
									if (strpbrk(classname,towbarData[i].tags) != NULL){


									}//if
								}//for

								VECTOR3 touchdown0,touchdown1,touchdown2;
								v->GetTouchdownPoints(touchdown0,touchdown1,touchdown2);

								double heightDiff = B1200_TOWBARATTACHFRONT_POS.y - B1200_COG -(childAttachpos.y-touchdown0.y) - 0.001;
								//Lengths
								towbarElevation = -asin(heightDiff/B1200_TOWBARLENGTH);
								//towbarElevation = 0;
								double towbarLength = B1200_TOWBARLENGTH*cos(towbarElevation);
								
								//Lengths
								lb0 = B1200_TOWBARATTACHBACK_POS.z;
								lf1 = towbarLength + (childAttachpos.z-touchdown0.z);
								//lf1 = -lf1;
								lb1 = 0.05;
								lf2 = -childAttachpos.z-(touchdown1.z+touchdown2.z)/2;

								//Angles
								theta2 = PI;

								return true;
							}//if attach
							else {
								return false;
							}//else attach
						}//if
					}//else
				}//for scan
			}// if in range
		}//vessel list
	}//else
	
	//If not returned before now, code has failed
	return false;
}//AttachPlane


//-------------------------------------------------------------------------
// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
// without scenery editor"
bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr=(char*)data;	if(*cPtr==2){*cPtr=3;strcpy(cPtr+2,str);}
	else if(*cPtr==4){*cPtr=5;strcpy(cPtr+42,str);}
	else if(*cPtr==6){*cPtr=7;strcpy(cPtr+82,str);}return true;
}
void JBT_B1200::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding==FALSE&&cAddUMmuToVessel[0]==0)
		return;
	if(bStartAdding==TRUE){
		int salut=sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel,0,sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0]=1;
	}
	else if(cAddUMmuToVessel[0]==1){
		cAddUMmuToVessel[0]=2;
		oapiOpenInputBox ("Enter new crew's name (or escape)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==3){
		cAddUMmuToVessel[0]=4;
		oapiOpenInputBox ("Enter crew's age",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==5){
		cAddUMmuToVessel[0]=6;
		oapiOpenInputBox ("Enter function (Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==7){
		cAddUMmuToVessel[0]=0;
		int Age=max(5,min(100,atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2],Age,70,70,&cAddUMmuToVessel[82])==TRUE){
			sprintf(SendHudMessage(),"Crew \"%s\" aged %i added to vessel",&cAddUMmuToVessel[2],Age);
		}
		else{
			strcpy(SendHudMessage(),"ERROR: Crew not added (vessel full?)");
		}
	}
}
// END of  "Allow user to add crew to your ship without scenery editor"
//-------------------------------------------------------------------------

// Set a mesh group visible or invisible
// Note: dwMeshGroup is 0-based
// Thanks to dbeachy1 for this one
void JBT_B1200::SetMeshGroupVisible(DEVMESHHANDLE hMesh, DWORD dwMeshGroup, bool isVisible)
{
	// Note: for details on mesh group flags, refer to page 7 of 3DModel.pdf.
	/*
	Mesh type   Flag        Interpretation
	---------   ----------  ----------------------------------------------------
	Vessel      0x00000001  Do not use this group to render ground shadows
	Vessel      0x00000002  Do not render this group
	Vessel      0x00000004  Do not apply lighting when rendering this group
	Vessel      0x00000008  Texture blending directive: additive with background
	*/

	GROUPEDITSPEC geSpec;
	memset(&geSpec, 0, sizeof(GROUPEDITSPEC));  // init all to zero
	geSpec.UsrFlag = 0x00000003;    // toggle shadows as well; this will be ANDed or ORd with the group's flags

	// ORG: MESHGROUP *pGroup = oapiMeshGroup(hMesh, dwMeshGroup);  // should never be null!
	if (isVisible)  // assignment separated for clarity
		// ORG: pGroup->UsrFlag &= ~renderFlag; // clear the "do not render" bits
		geSpec.flags = GRPEDIT_DELUSERFLAG;  // clear the "do not render" bits
	else
		// ORG: pGroup->UsrFlag |= renderFlag;  // set the "do not render" bits
		geSpec.flags = GRPEDIT_ADDUSERFLAG;  // set the "do not render" bits

	oapiEditMeshGroup(hMesh, dwMeshGroup, &geSpec);
}

void JBT_B1200::ChangeAttachment(ATTACHMENTHANDLE attachmentToChange,double elevation, double azimuth,int towbar){
	
	VECTOR3 currentPos,currentDir,currentRot;
	GetAttachmentParams(attachmentToChange,currentPos,currentDir,currentRot);
	VECTOR3 nextPos = currentPos;
	
	VECTOR3 nextDir = _V(0,0,1);
	VECTOR3 nextRot = _V(0,1,0);
	nextDir.y = sin(elevation); //Towbar depression angle
	//z = sin(altitude)
	double hyp = cos(elevation); //Towbar depression angle
	nextDir.z = hyp*cos(azimuth);
	nextDir.x = hyp*sin(azimuth);

	//Rotation normal to the dir
	nextRot.y = sin(elevation+(PI/2)); //Towbar depression angle
	//z = sin(altitude)
	hyp = cos(elevation+(PI/2)); //Towbar depression angle
	nextRot.z = hyp*cos(azimuth);
	nextRot.x = hyp*sin(azimuth);

	SetAttachmentParams(attachmentToChange, nextPos, nextDir, nextRot);

	switch(towbar) {
		
		case -2:GetAttachmentParams(back,currentPos,currentDir,currentRot);
				nextDir = currentDir;
				nextRot = currentRot;
		case -1:
				nextPos = nextPos + _V(B1200_TOWBARLENGTH*cos(elevation)*sin(azimuth),B1200_TOWBARLENGTH*sin(elevation),B1200_TOWBARLENGTH*cos(elevation)*cos(azimuth));
				//sprintf (oapiDebugString(), "%f %f %f",elevation,azimuth,nextPos.z);
				SetAttachmentParams(back, nextPos, nextDir, nextRot);
				break;
		
		case 2:	GetAttachmentParams(front,currentPos,currentDir,currentRot);
				nextDir = currentDir;
				nextRot = currentRot;
		case 1:
				nextPos = nextPos + _V(B1200_TOWBARLENGTH*cos(elevation)*sin(azimuth),B1200_TOWBARLENGTH*sin(elevation),B1200_TOWBARLENGTH*cos(elevation)*cos(azimuth));
				SetAttachmentParams(front, nextPos, nextDir, nextRot);
				break;
		// = _V((lf1+lb1) * sin(azimuth+PI) + currentPos.x ,currentPos.y,storedData.towbarLength*cos(towbarRelHeading+PI) + storedData.TowbarStartPos.z);
	}

	}
//sprintf (oapiDebugString(), "%f %f",distance,size);
//sprintf (oapiDebugString(), "%f %f %f",childGlobalPos.x,childGlobalPos.y,childGlobalPos.z);